==> eBackup-Upgrade-Plugin-8.3.0.zip: eBackup/scene/upgrade/hcs_update/params/params_check.py <==
# -*- coding: utf-8 -*-
import socket

import utils.common.log as logger
from utils.DBAdapter.DBConnector import BaseOps
from utils.business.manageone_cmdb_util import ManageOneCmdbUtil
from utils.business.param_util import HandleParamCheck
from utils.business.param_util import ParamUtil
from utils.common.exception import HCCIException

from plugins.eBackup.common.params_check_tool import check_datamover_web_login
from plugins.eBackup.common.params_check_tool import check_ebackup_nodes
from plugins.eBackup.common.params_check_tool import check_openstack_params
from plugins.eBackup.common.params_check_tool import init_params
from plugins.eBackup.common.util import Utils


def params_check(project_id):
    """插件参数校验接口函数

    错误信息返回值（需要中英文信息）：
    1、支持通过HCCIException(error_code, *arg, **kwargs)类获取中英文信息,此方法是获取公共方法中的错误
    2、支持通过code2Msg(error_code, *arg, **kwargs)函数获取中英文信息,此方法是获取各组件自定义的错误
    """
    ebackup_datamover_nodes = get_ebackup_datamover_nodes_from_cmdb(project_id)
    Utils.set_value_to_config_file("eBackup_Datamover_nodes", ebackup_datamover_nodes)
    db_opt = ParamUtil()
    check_data = db_opt.get_need_check_cloud_params(project_id, "eBackup")
    check_data.update({"eBackup_Datamover_nodes": ebackup_datamover_nodes})
    check_result = HandleParamCheck()
    all_params = ["openstack_cps_web_pwd", "openstack_cps_admin_pwd",
                  "eBackup_Datamover_nodes", "eBackup_hcp_pwd",
                  "eBackup_root_pwd", "eBackup_admin_pwd", "eBackup_dmk_user",
                  "eBackup_dmk_password"]
    init_params(check_data, db_opt, project_id, check_result, all_params)
    unpassed_ebackup_hcp_nodes = []
    unpassed_ebackup_root_nodes = []
    unpassed_openstack_fsp_nodes = []
    unpassed_openstack_root_nodes = []
    fsp_expire_msg = {}
    hcp_expire_msg = {}
    root_lock_msg = {}
    check_ebackup_nodes(
        check_result, "eBackup_Datamover_nodes",
        check_data.get("eBackup_Datamover_nodes"),
        check_data.get("eBackup_hcp_pwd"), check_data.get("eBackup_root_pwd"),
        unpassed_ebackup_hcp_nodes, unpassed_ebackup_root_nodes,
        hcp_expire_msg, root_lock_msg)
    param_check_judge(check_result, unpassed_ebackup_hcp_nodes, hcp_expire_msg,
                      root_lock_msg, unpassed_ebackup_root_nodes)
    check_openstack_params(
        check_result, check_data, unpassed_openstack_fsp_nodes,
        unpassed_openstack_root_nodes, fsp_expire_msg)
    ebackup_check_openstack_param(
        check_result, unpassed_openstack_fsp_nodes, fsp_expire_msg,
        unpassed_ebackup_hcp_nodes, unpassed_openstack_root_nodes)
    # check datamover web login
    result = check_datamover_web_login(check_data, check_result, all_params)
    return result


def param_check_judge(check_result, unpassed_ebackup_hcp_nodes, hcp_expire_msg,
                      root_lock_msg, unpassed_ebackup_root_nodes):
    if 0 != len(unpassed_ebackup_hcp_nodes) and 0 != len(hcp_expire_msg):
        result = HCCIException(650038, ",".join(unpassed_ebackup_hcp_nodes),
                              'hcp',
                              ','.join(list(hcp_expire_msg.keys())), 'hcp',
                              str(hcp_expire_msg))
        check_result.set_check_result(param_keys=["eBackup_hcp_pwd"],
                                      status=500, error_msg=result)
    elif 0 != len(hcp_expire_msg):
        check_result.set_check_result(
            param_keys=["eBackup_hcp_pwd"],
            status=500,
            error_msg=HCCIException(650037,
                                   ','.join(list(hcp_expire_msg.keys())),
                                   "hcp", str(hcp_expire_msg)))
    elif 0 != len(unpassed_ebackup_hcp_nodes):
        check_result.set_check_result(param_keys=["eBackup_hcp_pwd"],
                                      status=500,
                                      error_msg=HCCIException(650001, ",".join(
                                          unpassed_ebackup_hcp_nodes)))
    elif 0 != len(root_lock_msg):
        check_result.set_check_result(
            param_keys=["eBackup_root_pwd"],
            status=500,
            error_msg=HCCIException(650042,
                                   ','.join(list(root_lock_msg.keys())),
                                   "root", str(root_lock_msg)))
    elif 0 == len(unpassed_ebackup_root_nodes):
        check_result.set_check_result(param_keys=["eBackup_root_pwd"],
                                      status=200, error_msg="")
    else:
        check_result.set_check_result(param_keys=["eBackup_root_pwd"],
                                      status=500,
                                      error_msg=HCCIException(650002, ",".join(
                                          unpassed_ebackup_root_nodes)))


def ebackup_check_openstack_param(check_result,
                                  unpassed_openstack_fsp_nodes,
                                  fsp_expire_msg,
                                  unpassed_ebackup_hcp_nodes,
                                  unpassed_openstack_root_nodes):
    if 0 != len(unpassed_openstack_fsp_nodes) and 0 != len(fsp_expire_msg):
        result = HCCIException(650038, ",".join(unpassed_ebackup_hcp_nodes),
                              'fsp',
                              ','.join(list(fsp_expire_msg.keys())), 'fsp',
                              str(fsp_expire_msg))
        check_result.set_check_result(param_keys=["openstack_cps_admin_pwd"],
                                      status=500, error_msg=result)
    elif 0 != len(fsp_expire_msg):
        check_result.set_check_result(
            param_keys=["openstack_cps_admin_pwd"], status=500,
            error_msg=HCCIException(
                650037, ','.join(list(fsp_expire_msg.keys())), "fsp",
                str(fsp_expire_msg)))
    elif 0 != len(unpassed_openstack_fsp_nodes):
        check_result.set_check_result(
            param_keys=["openstack_cps_admin_pwd"], status=500,
            error_msg=HCCIException(
                650005, ",".join(unpassed_openstack_fsp_nodes)))

    if 0 != len(unpassed_openstack_root_nodes):
        check_result.set_check_result(
            param_keys=["openstack_cps_admin_pwd"], status=500,
            error_msg=HCCIException(
                650006, ",".join(unpassed_openstack_root_nodes)))


def check_ssh_port(ip_address):
    ssh_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        logger.info(f"check ssh port on node {ip_address}")
        result = ssh_socket.connect_ex((ip_address, 22))
    except Exception as err:
        logger.error(f"Failed to connect node{ip_address} 22 port. "
                    f"reason {str(repr(err))}.")
        result = 1
    finally:
        ssh_socket.close()
    return not bool(result)


def get_node_manage_ip(ip_address):
    if len(ip_address) == 1:
        return ip_address[0].get('ip')
    for _ip in ip_address:
        if check_ssh_port(_ip.get('ip')):
            return _ip.get('ip')
    return ip_address[0].get('ip')


def get_ebackup_datamover_nodes_from_cmdb(project_id):
    ebackup_datamover_nodes_dict = {}
    data = ParamUtil().get_need_check_cloud_params(
        project_id, "ManageOne_public_params")
    current_region_id = BaseOps().get_regionId_by_projectId(project_id)[0]
    cmdb_ip = data.get("cmdb_ip")
    cmdb_password = data.get("cmdb_password")
    cmdb_account = data.get("cmdb_account")
    cmdb = ManageOneCmdbUtil.get_instance(cmdb_ip, cmdb_password,
                                          cmdb_account, project_id)
    cmdb_nodes_list = cmdb.get_deploy_node_info(current_region_id)
    if not cmdb_nodes_list:
        logger.error("Query cmdb nodes info failed.")
        raise Exception("Query cmdb nodes info failed.")
    for node in cmdb_nodes_list:
        if node["name"].find("eBackup_service") != -1:
            group_name = node.get("name")[:-1] + 's'
            manage_ip = get_node_manage_ip(node.get('ipAddresses'))
            if group_name not in ebackup_datamover_nodes_dict:
                ebackup_datamover_nodes_dict.update({group_name: [manage_ip]})
            else:
                ebackup_datamover_nodes_dict[group_name].append(manage_ip)
        else:
            logger.info(f"Skip non-ebackup nodes{node.get('name')} info.")
    if not ebackup_datamover_nodes_dict:
        logger.error("Get ebackup datamover nodes info failed.")
        raise Exception("Get ebackup datamover nodes info failed.")
    return '|'.join([';'.join(nodes) for nodes in ebackup_datamover_nodes_dict.values()])

==> eBackup-Upgrade-Plugin-8.3.1.zip: eBackup/scene/upgrade/hcs_update/params/params_check.py <==
# -*- coding: utf-8 -*-
import socket

import utils.common.log as logger
from utils.DBAdapter.DBConnector import BaseOps
from utils.business.manageone_cmdb_util import ManageOneCmdbUtil
from utils.business.param_util import HandleParamCheck
from utils.business.param_util import ParamUtil
from utils.common.exception import HCCIException

from plugins.eBackup.common.params_check_tool import check_datamover_web_login, param_check_judge
from plugins.eBackup.common.params_check_tool import check_ebackup_nodes
from plugins.eBackup.common.params_check_tool import check_openstack_params
from plugins.eBackup.common.params_check_tool import init_params
from plugins.eBackup.common.model import SshParamsCheckInfo, SshInfo
from plugins.eBackup.common.util import Utils


def params_check(project_id):
    """插件参数校验接口函数

    错误信息返回值（需要中英文信息）：
    1、支持通过HCCIException(error_code, *arg, **kwargs)类获取中英文信息,此方法是获取公共方法中的错误
    2、支持通过code2Msg(error_code, *arg, **kwargs)函数获取中英文信息,此方法是获取各组件自定义的错误
    """
    ebackup_datamover_nodes = get_ebackup_datamover_nodes_from_cmdb(project_id)
    Utils.set_value_to_config_file("eBackup_Datamover_nodes", ebackup_datamover_nodes)
    db_opt = ParamUtil()
    check_data = db_opt.get_need_check_cloud_params(project_id, "eBackup")
    check_data.update({"eBackup_Datamover_nodes": ebackup_datamover_nodes})
    check_result = HandleParamCheck()
    all_params = ["openstack_cps_web_pwd", "openstack_cps_admin_pwd",
                  "eBackup_Datamover_nodes", "eBackup_hcp_pwd",
                  "eBackup_root_pwd", "eBackup_admin_pwd", "eBackup_dmk_user",
                  "eBackup_dmk_password"]
    init_params(check_data, db_opt, project_id, check_result, all_params)

    ebackup_abnormal_nodes_info = SshParamsCheckInfo()
    openstack_abnormal_nodes_info = SshParamsCheckInfo()
    ip_list = check_data.get("eBackup_Datamover_nodes").replace(" ", "").replace("|", ";").split(";")
    ssh_info = SshInfo(ip_list, "hcp", check_data.get("eBackup_hcp_pwd"), check_data.get("eBackup_root_pwd"))
    check_ebackup_nodes(check_result, "eBackup_Datamover_nodes", ssh_info, ebackup_abnormal_nodes_info)

    param_check_judge(check_result, ebackup_abnormal_nodes_info)
    check_openstack_params(check_result, check_data, openstack_abnormal_nodes_info)
    ebackup_check_openstack_param(
        check_result, ebackup_abnormal_nodes_info, openstack_abnormal_nodes_info)
    # check datamover web login
    result = check_datamover_web_login(check_data, check_result, all_params)
    return result


def ebackup_check_openstack_param(check_result, ebackup_abnormal_nodes_info, openstack_abnormal_nodes_info):

    unpassed_ebackup_hcp_nodes = ebackup_abnormal_nodes_info.ssh_user_fail_nodes
    unpassed_openstack_fsp_nodes = openstack_abnormal_nodes_info.ssh_user_fail_nodes
    fsp_expire_msg = openstack_abnormal_nodes_info.ssh_user_fail_reason
    unpassed_openstack_root_nodes = openstack_abnormal_nodes_info.sudo_user_fail_nodes
    if 0 != len(unpassed_openstack_fsp_nodes) and 0 != len(fsp_expire_msg):
        result = HCCIException(650038, ",".join(unpassed_ebackup_hcp_nodes), 'fsp',
                               ','.join(list(fsp_expire_msg.keys())), 'fsp', str(fsp_expire_msg))
        check_result.set_check_result(param_keys=["openstack_cps_admin_pwd"], status=500, error_msg=result)
    elif 0 != len(fsp_expire_msg):
        check_result.set_check_result(param_keys=["openstack_cps_admin_pwd"], status=500,
                                      error_msg=HCCIException(650037, ','.join(list(fsp_expire_msg.keys())), "fsp",
                                                              str(fsp_expire_msg)))
    elif 0 != len(unpassed_openstack_fsp_nodes):
        check_result.set_check_result(param_keys=["openstack_cps_admin_pwd"], status=500, error_msg=HCCIException(
            650005, ",".join(unpassed_openstack_fsp_nodes)))

    if 0 != len(unpassed_openstack_root_nodes):
        check_result.set_check_result(param_keys=["openstack_cps_admin_pwd"], status=500, error_msg=HCCIException(
            650006, ",".join(unpassed_openstack_root_nodes)))


def check_ssh_port(ip_address):
    ssh_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        logger.info(f"check ssh port on node {ip_address}")
        result = ssh_socket.connect_ex((ip_address, 22))
    except Exception as err:
        logger.error(f"Failed to connect node{ip_address} 22 port. "
                    f"reason {str(repr(err))}.")
        result = 1
    finally:
        ssh_socket.close()
    return not bool(result)


def get_node_manage_ip(ip_address):
    if len(ip_address) == 1:
        return ip_address[0].get('ip')
    for _ip in ip_address:
        if check_ssh_port(_ip.get('ip')):
            return _ip.get('ip')
    return ip_address[0].get('ip')


def get_ebackup_datamover_nodes_from_cmdb(project_id):
    ebackup_datamover_nodes_dict = {}
    data = ParamUtil().get_need_check_cloud_params(
        project_id, "ManageOne_public_params")
    current_region_id = BaseOps().get_regionId_by_projectId(project_id)[0]
    cmdb_ip = data.get("cmdb_ip")
    cmdb_password = data.get("cmdb_password")
    cmdb_account = data.get("cmdb_account")
    cmdb = ManageOneCmdbUtil.get_instance(cmdb_ip, cmdb_password,
                                          cmdb_account, project_id)
    cmdb_nodes_list = cmdb.get_deploy_node_info(current_region_id)
    if not cmdb_nodes_list:
        logger.error("Query cmdb nodes info failed.")
        raise Exception("Query cmdb nodes info failed.")
    for node in cmdb_nodes_list:
        if node["name"].find("eBackup_service") != -1:
            group_name = node.get("name")[:-1] + 's'
            manage_ip = get_node_manage_ip(node.get('ipAddresses'))
            if group_name not in ebackup_datamover_nodes_dict:
                ebackup_datamover_nodes_dict.update({group_name: [manage_ip]})
            else:
                ebackup_datamover_nodes_dict[group_name].append(manage_ip)
        else:
            logger.info(f"Skip non-ebackup nodes{node.get('name')} info.")
    if not ebackup_datamover_nodes_dict:
        logger.error("Get ebackup datamover nodes info failed.")
        raise Exception("Get ebackup datamover nodes info failed.")
    return '|'.join([';'.join(nodes) for nodes in ebackup_datamover_nodes_dict.values()])
